﻿using System;
using System.Collections.Generic;
using System.Runtime.InteropServices;
using System.Text;

namespace ConsoleApp1
{
    // https://stackoverflow.com/questions/58814332/read-raw-data-from-usbpcap-library
    [StructLayout(LayoutKind.Sequential, Pack = 1)]
    public struct pcap_hdr_t
    {
        public uint magic_number;   /* magic number */
        public ushort version_major;  /* major version number */
        public ushort version_minor;  /* minor version number */
        public int thiszone;       /* GMT to local correction */
        public uint sigfigs;        /* accuracy of timestamps */
        public uint snaplen;        /* max length of captured packets, in octets */
        public uint network;        /* data link type */
    };

    [StructLayout(LayoutKind.Sequential, Pack = 1)]
    public struct pcaprec_hdr_t
    {
        public uint ts_sec;         /* timestamp seconds */
        public uint ts_usec;        /* timestamp microseconds */
        public uint incl_len;       /* number of octets of packet saved in file */
        public uint orig_len;       /* actual length of packet */
    }

    // https://desowin.org/usbpcap/captureformat.html
    [StructLayout(LayoutKind.Sequential, Pack = 1)]
    public struct USBPCAP_BUFFER_PACKET_HEADER
    {
        public ushort headerLen; /* This header length */
        public ulong irpId;     /* I/O Request packet ID */
        public USBD_STATUS status;    /* USB status code
                                   (on return from host controller) */
        public URB_FUNCTION function;  /* URB Function */
        public byte info;      /* I/O Request info */

        public ushort bus;       /* bus (RootHub) number */
        public ushort device;    /* device address */
        public byte endpoint;  /* endpoint number and transfer direction */
        public byte transfer;  /* transfer type */

        public uint dataLength;/* Data length */

        public IRPDierction Direction
        {
            get
            {
                if ((this.info & 0x01) == 0x0)
                {
                    return IRPDierction.FDO_TO_PDO;
                }
                else
                {
                    return IRPDierction.PDO_TO_FDO;
                }
            }
        }
    }

    public enum IRPDierction
    {
        FDO_TO_PDO = 0,
        PDO_TO_FDO = 1,
    }

    // https://github.com/tpn/winsdk-10/blob/master/Include/10.0.14393.0/shared/usb.h
    public enum URB_FUNCTION : ushort
    {
        URB_FUNCTION_SELECT_CONFIGURATION = 0x0000,
        URB_FUNCTION_SELECT_INTERFACE = 0x0001,
        URB_FUNCTION_ABORT_PIPE = 0x0002,
        URB_FUNCTION_TAKE_FRAME_LENGTH_CONTROL = 0x0003,
        URB_FUNCTION_RELEASE_FRAME_LENGTH_CONTROL = 0x0004,
        URB_FUNCTION_GET_FRAME_LENGTH = 0x0005,
        URB_FUNCTION_SET_FRAME_LENGTH = 0x0006,
        URB_FUNCTION_GET_CURRENT_FRAME_NUMBER = 0x0007,
        URB_FUNCTION_CONTROL_TRANSFER = 0x0008,
        URB_FUNCTION_BULK_OR_INTERRUPT_TRANSFER = 0x0009,
        URB_FUNCTION_ISOCH_TRANSFER = 0x000A,
        URB_FUNCTION_GET_DESCRIPTOR_FROM_DEVICE = 0x000B,
        URB_FUNCTION_SET_DESCRIPTOR_TO_DEVICE = 0x000C,
        URB_FUNCTION_SET_FEATURE_TO_DEVICE = 0x000D,
        URB_FUNCTION_SET_FEATURE_TO_INTERFACE = 0x000E,
        URB_FUNCTION_SET_FEATURE_TO_ENDPOINT = 0x000F,
        URB_FUNCTION_CLEAR_FEATURE_TO_DEVICE = 0x0010,
        URB_FUNCTION_CLEAR_FEATURE_TO_INTERFACE = 0x0011,
        URB_FUNCTION_CLEAR_FEATURE_TO_ENDPOINT = 0x0012,
        URB_FUNCTION_GET_STATUS_FROM_DEVICE = 0x0013,
        URB_FUNCTION_GET_STATUS_FROM_INTERFACE = 0x0014,
        URB_FUNCTION_GET_STATUS_FROM_ENDPOINT = 0x0015,
        URB_FUNCTION_RESERVED_0X0016 = 0x0016,
        URB_FUNCTION_VENDOR_DEVICE = 0x0017,
        URB_FUNCTION_VENDOR_INTERFACE = 0x0018,
        URB_FUNCTION_VENDOR_ENDPOINT = 0x0019,
        URB_FUNCTION_CLASS_DEVICE = 0x001A,
        URB_FUNCTION_CLASS_INTERFACE = 0x001B,
        URB_FUNCTION_CLASS_ENDPOINT = 0x001C,
        URB_FUNCTION_RESERVE_0X001D = 0x001D,         // previously URB_FUNCTION_RESET_PIPE
        URB_FUNCTION_SYNC_RESET_PIPE_AND_CLEAR_STALL = 0x001E,
        URB_FUNCTION_CLASS_OTHER = 0x001F,
        URB_FUNCTION_VENDOR_OTHER = 0x0020,
        URB_FUNCTION_GET_STATUS_FROM_OTHER = 0x0021,
        URB_FUNCTION_CLEAR_FEATURE_TO_OTHER = 0x0022,
        URB_FUNCTION_SET_FEATURE_TO_OTHER = 0x0023,
        URB_FUNCTION_GET_DESCRIPTOR_FROM_ENDPOINT = 0x0024,
        URB_FUNCTION_SET_DESCRIPTOR_TO_ENDPOINT = 0x0025,
        URB_FUNCTION_GET_CONFIGURATION = 0x0026,
        URB_FUNCTION_GET_INTERFACE = 0x0027,
        URB_FUNCTION_GET_DESCRIPTOR_FROM_INTERFACE = 0x0028,
        URB_FUNCTION_SET_DESCRIPTOR_TO_INTERFACE = 0x0029,

        URB_FUNCTION_GET_MS_FEATURE_DESCRIPTOR = 0x002A,
        URB_FUNCTION_SYNC_RESET_PIPE = 0x0030,
        URB_FUNCTION_SYNC_CLEAR_STALL = 0x0031,
        URB_FUNCTION_CONTROL_TRANSFER_EX = 0x0032,
        URB_FUNCTION_RESERVE_0X0033 = 0x0033,
        URB_FUNCTION_RESERVE_0X0034 = 0x0034,
        URB_FUNCTION_OPEN_STATIC_STREAMS = 0x0035,
        URB_FUNCTION_CLOSE_STATIC_STREAMS = 0x0036,
        URB_FUNCTION_BULK_OR_INTERRUPT_TRANSFER_USING_CHAINED_MDL = 0x0037,
        URB_FUNCTION_ISOCH_TRANSFER_USING_CHAINED_MDL = 0x0038,
        URB_FUNCTION_RESERVE_0X002B = 0x002B,
        URB_FUNCTION_RESERVE_0X002C = 0x002C,
        URB_FUNCTION_RESERVE_0X002D = 0x002D,
        URB_FUNCTION_RESERVE_0X002E = 0x002E,
        URB_FUNCTION_RESERVE_0X002F = 0x002F,
    }

    public enum USBD_STATUS : uint
    {
        USBD_STATUS_SUCCESS = 0x00000000,
        USBD_STATUS_PORT_OPERATION_PENDING = 0x00000001,
        USBD_STATUS_PENDING = 0x40000000,

        USBD_STATUS_CRC = 0xC0000001,
        USBD_STATUS_BTSTUFF = 0xC0000002,
        USBD_STATUS_DATA_TOGGLE_MISMATCH = 0xC0000003,
        USBD_STATUS_STALL_PID = 0xC0000004,
        USBD_STATUS_DEV_NOT_RESPONDING = 0xC0000005,
        USBD_STATUS_PID_CHECK_FAILURE = 0xC0000006,
        USBD_STATUS_UNEXPECTED_PID = 0xC0000007,
        USBD_STATUS_DATA_OVERRUN = 0xC0000008,
        USBD_STATUS_DATA_UNDERRUN = 0xC0000009,
        USBD_STATUS_RESERVED1 = 0xC000000A,
        USBD_STATUS_RESERVED2 = 0xC000000B,
        USBD_STATUS_BUFFER_OVERRUN = 0xC000000C,
        USBD_STATUS_BUFFER_UNDERRUN = 0xC000000D,
        USBD_STATUS_NOT_ACCESSED = 0xC000000F,
        USBD_STATUS_FIFO = 0xC0000010,
        USBD_STATUS_XACT_ERROR = 0xC0000011,
        USBD_STATUS_BABBLE_DETECTED = 0xC0000012,
        USBD_STATUS_DATA_BUFFER_ERROR = 0xC0000013,
        USBD_STATUS_NO_PING_RESPONSE = 0xC0000014,
        USBD_STATUS_INVALID_STREAM_TYPE = 0xC0000015,
        USBD_STATUS_INVALID_STREAM_ID = 0xC0000016,
        USBD_STATUS_ENDPOINT_HALTED = 0xC0000030,

        USBD_STATUS_INVALID_URB_FUNCTION = 0x80000200,
        USBD_STATUS_INVALID_PARAMETER = 0x80000300,
        USBD_STATUS_ERROR_BUSY = 0x80000400,
        USBD_STATUS_INVALID_PIPE_HANDLE = 0x80000600,
        USBD_STATUS_NO_BANDWIDTH = 0x80000700,
        USBD_STATUS_INTERNAL_HC_ERROR = 0x80000800,
        USBD_STATUS_ERROR_SHORT_TRANSFER = 0x80000900,
        USBD_STATUS_BAD_START_FRAME = 0xC0000A00,
        USBD_STATUS_ISOCH_REQUEST_FAILED = 0xC0000B00,
        USBD_STATUS_FRAME_CONTROL_OWNED = 0xC0000C00,
        USBD_STATUS_FRAME_CONTROL_NOT_OWNED = 0xC0000D00,
        USBD_STATUS_NOT_SUPPORTED = 0xC0000E00,
        USBD_STATUS_INVALID_CONFIGURATION_DESCRIPTOR = 0xC0000F00,
        USBD_STATUS_INSUFFICIENT_RESOURCES = 0xC0001000,
        USBD_STATUS_SET_CONFIG_FAILED = 0xC0002000,
        USBD_STATUS_BUFFER_TOO_SMALL = 0xC0003000,
        USBD_STATUS_INTERFACE_NOT_FOUND = 0xC0004000,
        USBD_STATUS_INAVLID_PIPE_FLAGS = 0xC0005000,
        USBD_STATUS_TIMEOUT = 0xC0006000,
        USBD_STATUS_DEVICE_GONE = 0xC0007000,
        USBD_STATUS_STATUS_NOT_MAPPED = 0xC0008000,
        USBD_STATUS_HUB_INTERNAL_ERROR = 0xC0009000,
        USBD_STATUS_CANCELED = 0xC0010000,
        USBD_STATUS_ISO_NOT_ACCESSED_BY_HW = 0xC0020000,
        USBD_STATUS_ISO_TD_ERROR = 0xC0030000,
        USBD_STATUS_ISO_NA_LATE_USBPORT = 0xC0040000,
        USBD_STATUS_ISO_NOT_ACCESSED_LATE = 0xC0050000,
        USBD_STATUS_BAD_DESCRIPTOR = 0xC0100000,
        USBD_STATUS_BAD_DESCRIPTOR_BLEN = 0xC0100001,
        USBD_STATUS_BAD_DESCRIPTOR_TYPE = 0xC0100002,
        USBD_STATUS_BAD_INTERFACE_DESCRIPTOR = 0xC0100003,
        USBD_STATUS_BAD_ENDPOINT_DESCRIPTOR = 0xC0100004,
        USBD_STATUS_BAD_INTERFACE_ASSOC_DESCRIPTOR = 0xC0100005,
        USBD_STATUS_BAD_CONFIG_DESC_LENGTH = 0xC0100006,
        USBD_STATUS_BAD_NUMBER_OF_INTERFACES = 0xC0100007,
        USBD_STATUS_BAD_NUMBER_OF_ENDPOINTS = 0xC0100008,
        USBD_STATUS_BAD_ENDPOINT_ADDRESS = 0xC0100009,
    }
}
